﻿import json
from base64 import b64encode, urlsafe_b64decode
from datetime import datetime
import logging

from requests import Session
from ladon.ladonizer import ladonize
from ladon.types.ladontype import LadonType

import clr
clr.AddReference('System.Security.Cryptography.X509Certificates')
clr.AddReference('System.Security.Cryptography.Pkcs')

from System.Security.Cryptography.X509Certificates import X509Store, StoreLocation, OpenFlags
from System.Security.Cryptography.Pkcs import ContentInfo, SignedCms, CmsSigner

BASE_URLS = {
    True: 'https://ismp.crpt.ru/api',
    False: 'https://sandbox.crptech.ru/api',
}

def get_cert_by_name(cert_name):
    store = X509Store(StoreLocation.CurrentUser)
    store.Open(OpenFlags.ReadOnly)
    certCollection = store.Certificates
    store.Close()

    cert = None
    for c in certCollection:
        if c.Subject == cert_name or c.Subject+"#"+c.FriendlyName+"#"+str(c.NotAfter) == cert_name:
            cert = c
            break

    return cert


def sign(data, signingCert, detached = False):
    content = ContentInfo(data)
    signedMessage = SignedCms(content, detached)

    signer = CmsSigner(signingCert)
    signedMessage.ComputeSignature(signer)
    signedBytes = signedMessage.Encode()
    return bytes(signedBytes)


class AuthResponse(LadonType):
    token = str
    expire = str
    error = str


class CrptAuth:

    @ladonize(rtype=str)
    def get_cert_list(self):
        res = []

        store = X509Store(StoreLocation.CurrentUser)
        store.Open(OpenFlags.ReadOnly)
        certCollection = store.Certificates
        store.Close()

        for c in certCollection:
            res.append(c.Subject+"#"+c.FriendlyName+"#"+str(c.NotAfter))

        return '|'.join(res)

    @ladonize(bool, str, rtype=AuthResponse)
    def get_token(self, is_prod, cert_name):
        res = AuthResponse()
        res.token = ''
        res.expire_date = ''
        res.error = ''

        cert = get_cert_by_name(cert_name)
        if not cert:
            res.error = 'Сертификат не найден'
            return res

        s = Session()
        sres = s.get(f'{BASE_URLS[is_prod]}/v3/auth/cert/key')
        sresp = sres.json()

        data = sresp['data'].encode('ascii')
        signed_data = sign(data, cert)
        signed_data = b64encode(signed_data).decode('ascii')

        req = {
            'uuid': sresp['uuid'],
            'data': signed_data,
        }
        s.headers.update({'Authorization': 'Bearer'})
        sres = s.post(f'{BASE_URLS[is_prod]}/v3/auth/cert/', json=req)
        if sres.status_code != 200:
            res.error = f"Ошибка авторизации {sres.status_code} {sres.text}"
            return res

        token: str = sres.json()['token']
        parts = token.split('.')
        payload = json.loads(urlsafe_b64decode(parts[1]+"==="))
        expire = payload['exp']
        expire_time = datetime.fromtimestamp(expire)

        res.token = token
        res.expire = f'{expire_time:%Y%m%d}000000'
        return res

    @ladonize(bool, str, str, rtype=AuthResponse)
    def get_client_token(self, is_prod, cert_name, oms_id):
        res = AuthResponse()
        res.token = ''
        res.expire_date = ''
        res.error = ''

        cert = get_cert_by_name(cert_name)
        if not cert:
            res.error = 'Сертификат не найден'
            return res

        s = Session()
        sres = s.get(f'{BASE_URLS[is_prod]}/v3/auth/cert/key')
        sresp = sres.json()

        data = sresp['data'].encode('ascii')
        signed_data = sign(data, cert)
        signed_data = b64encode(signed_data).decode('ascii')

        req = {
            'uuid': sresp['uuid'],
            'data': signed_data,
        }
        s.headers.update({'Authorization': 'Bearer'})
        sres = s.post(f'{BASE_URLS[is_prod]}/v3/auth/cert/'+oms_id, json=req)
        if sres.status_code != 200:
            res.error = f"Ошибка авторизации {sres.status_code} {sres.text}"
            return res

        
        
        token: str = sres.json()['token']
        """
        parts = token.split('.')
        payload = json.loads(urlsafe_b64decode(parts[1]+"==="))
        expire = payload['exp']
        expire_time = datetime.fromtimestamp(expire)
        res.token = token
        res.expire = f'{expire_time:%Y%m%d}000000'
        """

        res.token = token
        res.expire = '' #20200101000000'
        

        return res

    @ladonize(str, str, rtype=str)
    def sign_body(self, cert_name, body):
        cert = get_cert_by_name(cert_name)
        if not cert:
            return ''

        data = body.encode('utf8')
        signed = sign(data, cert, True)

        res = b64encode(signed).decode('ascii')
        return res

    @ladonize(str, str, rtype=str)
    def sign_body_clear(self, cert_name, body):
        body = json.dumps(json.loads(body))
        return self.sign_body(cert_name, body)

    @ladonize(str, rtype=str)
    def b64encode(self, body):
        body = json.dumps(json.loads(body))
        res = b64encode(body.encode('utf8'))
        return res.decode('ascii')

    @ladonize(rtype=str)
    def get_schema(self):
        with open("schema.json", 'rt') as f:
            return f.read()


if __name__ == '__main__':
    from ladon.server.wsgi import LadonWSGIApplication
    import wsgiref.simple_server

    application = LadonWSGIApplication([], [])

    port = 8008
    args = ('', port, application)
    server = wsgiref.simple_server.make_server(*args)

    logger = logging.getLogger('ladonlogger')
    logger.setLevel(logging.ERROR)

    print(f'Server started at {port}')
    try:
        server.serve_forever()
    except KeyboardInterrupt:
        pass
